"""
日志级别模块
Log level module

"""
# encoding: utf-8
# python 3.13.5

from typing import Optional, Any
from ..utils.fmt import fmt_file, fmt_message, fmt_console
from ..utils.configs import get_config, set_config
from ..utils.time import get_asctime
import pathlib

def _get_full_path(file_path, file_name):
    file_path.mkdir(parents=True, exist_ok=True)
    return file_path / file_name

file_path = pathlib.Path(get_config("file_path"))
file_name = get_config("file_name")
file_format = get_config("file_format")
file_encoding = get_config("file_encoding")
is_enable_console = get_config("enable_console")
is_enable_file = get_config("enable_file")

class Logger:
    def __init__(self):
        if pathlib.Path(file_path).exists():
            if not pathlib.Path(file_path).is_dir():
                self.warn("日志文件路径不是目录，已自动自动使用默认目录")
                set_config("file_path", "./logs")
                pathlib.Path("./logs").mkdir(parents=True, exist_ok=True)
        if _get_full_path(file_path, file_name).exists():
            from os import rename
            rename(_get_full_path(file_path, file_name), _get_full_path(file_path, f"{get_asctime().replace(':', '-')}.log"))
            self.debug("日志文件已存在，已自动重命名")
    
    def _log(self, msg, pf, lvn):
        if is_enable_file:
            with open(_get_full_path(file_path, file_name), "a", encoding=file_encoding) as f:
                f.write(fmt_file(lvn, fmt_message(msg, no_placeholder=True), pf))
        if is_enable_console:
            print(fmt_console(lvn, fmt_message(msg, no_placeholder=True), pf))
        return fmt_console(lvn, fmt_message(msg, no_placeholder=True), pf)

    def debug(self, message: Any, prefix: str | None = None, level: int = 0) -> Optional[str]:
        """
        调试日志
        Debug log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(0~9)
        """
        return self._log(message, prefix, level)

    def info(self, message: Any, prefix: str | None = None, level: int = 10) -> Optional[str]:
        """
        信息日志
        Info log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(10~19)
        """
        return self._log(message, prefix, level)

    def warn(self, message: Any, prefix: str | None = None, level: int = 20) -> Optional[str]:
        """
        警告日志
        Warn log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(20~29)
        """
        return self._log(message, prefix, level)

    def error(self, message: Any, prefix: str | None = None, level: int = 30) -> Optional[str]:
        """
        错误日志
        Error log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(30~39)
        """
        return self._log(message, prefix, level)

    def critical(self, message: Any, prefix: str | None = None, level: int = 40) -> Optional[str]:
        """
        严重错误日志
        Critical error log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(40~49)
        """
        return self._log(message, prefix, level)

    def log(self, message: Any, prefix: str | None = None, level: int = 50) -> Optional[str]:
        """
        自定义日志
        Custom log
        :param message: 消息内容 Message content
        :param prefix: 前缀 Prefix
        :param level: 日志级别 Log level(50~59...)
        """
        return self._log(message, prefix, level)


"""
时间工具模块,用于时间格式化和输出,缓存时间格式化结果
Time utility module, used for time formatting and output, caching time formatting results

"""
# encoding = utf-8
# python 3.13.5

from datetime import datetime
from .configs import get_config

import time

cache_time: str = ""
cache_time_ts: float = 0.0
cache_fmt: str | None = None

def get_asctime() -> str:
    """
    获取当前时间(YYYY-MM-DD HH:MM:SS),并缓存格式化结果
    Get current time(YYYY-MM-DD HH:MM:SS) and cache formatted result
    :return: 格式化后的时间 Formatted time
    """
    return _get_time(get_config("asctime_format"))

def get_time() -> str:
    """
    获取当前时间(HH:MM:SS),并缓存格式化结果
    Get current time(HH:MM:SS) and cache formatted result
    :return: 格式化后的时间 Formatted time
    """
    return _get_time(get_config("time_format"))

def get_date() -> str:
    """
    获取当前日期(YYYY-MM-DD),并缓存格式化结果
    Get current date(YYYY-MM-DD) and cache formatted result
    :return: 格式化后的日期 Formatted date
    """
    return _get_time(get_config("date_format"))

def get_weekday() -> str:
    """
    获取当前日期(星期几),并缓存格式化结果
    Get current date(weekday) and cache formatted result
    :return: 格式化后的星期几 Formatted weekday
    """
    return _get_time(get_config("weekday_format"))

def _get_time(fmt: str) -> str:
    """
    获取当前时间(根据指定格式),并缓存格式化结果
    Get current time(based on specified format) and cache formatted result
    :param fmt: 时间格式 Time format
    :return: 格式化后的时间 Formatted time
    """
    global cache_time, cache_time_ts, cache_fmt
    if cache_fmt is None:
        cache_fmt = fmt
    now = time.time()
    if cache_time and (now - cache_time_ts < 1) and (cache_fmt == fmt):
        return cache_time
    cache_time = datetime.now().strftime(fmt)
    cache_time_ts = now
    return cache_time


"""
py-logiliteal的配置设置,用于设置py-logiliteal的全局配置
py-logiliteal's config settings, used to set py-logiliteal's global config

"""
# encoding = utf-8
# python 3.13.5

import json
from os import remove
import shutil
from pathlib import Path
from typing import Union, Optional, Tuple
from logging import error

DEFAULT_CONFIG_PATH = "logger_config.json"
DEFAULT_CONFIG = {
    "file_level": "DEBUG",
    "file_name": "latest.log",
    "file_path": "./logs",
    "file_format": "{asctime} {levelname} | {prefix}{message}",
    "file_encoding": "utf-8",
    "enable_console": True,
    "enable_file": True,
    "console_color": True,
    "console_level": "INFO",
    "console_format": "{time} {levelname} | {prefix}{message}",
    "console_prefix": "Auto",
    "console_encoding": "utf-8",
    "asctime_format": "%Y-%m-%d %H:%M:%S",
    "time_format": "%H:%M:%S",
    "date_format": "%Y-%m-%d",
    "weekday_format": "%A",
    "level_name": {"DEBUG": "DEBUG", "INFO": "INFO", "WARN": "WARN", "ERRO": "ERRO", "CRIT": "CRIT"},
    "level_color": {"DEBUG": "#c1d5ff", "INFO": "#c1ffff", "WARN": "#fff600", "ERRO": "#ffa000", "CRIT": "#ff8181"},
}

g_config_cache = None
g_config_mtime = 0

def create_backup(config_path: Path) -> Tuple[bool, str]:
    """
    创建配置文件备份
    :param config_path: 配置文件路径
    :param backup_prefix: 备份文件前缀
    :return: (是否成功, 备份路径或错误信息)
    """
    try:
        if not config_path.exists():
            return False, f"配置文件不存在: {config_path}"
        backup_path = config_path.parent / f"logger_config_backup.json"

        if backup_path.exists():
            remove(backup_path)
        shutil.copy2(config_path, backup_path)
        return True, str(backup_path)
    except PermissionError:
        return False, f"权限不足，无法创建备份: {config_path}"
    except Exception as e:
        return False, f"备份失败: {str(e)}"

def handle_config_exceptions(func):
    """
    配置操作异常处理装饰器
    """
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except json.JSONDecodeError as e:
            error(f"配置文件格式错误: {e}")
            return False, f"配置文件格式错误: {str(e)}"
        except PermissionError:
            error(f"没有权限操作配置文件: {DEFAULT_CONFIG_PATH}")
            return False, "没有权限操作配置文件"
        except Exception as e:
            error(f"配置操作失败: {e}")
            return False, f"配置操作失败: {str(e)}"
    return wrapper

def get_config(select: str = None) -> Union[dict, str, bool, int, None]:
    """
    获取配置信息 Get config info
    :param select: 配置项名称 Config item name
    :return: 配置项值 Config item value
    """
    global g_config_cache, g_config_mtime
    config_path = Path(DEFAULT_CONFIG_PATH)

    if config_path.exists():
        current_mtime = config_path.stat().st_mtime
        if current_mtime != g_config_mtime or g_config_cache is None:
            with open(config_path, "r", encoding="utf-8") as f:
                g_config_cache = json.load(f)
                g_config_mtime = current_mtime
    else:
        with open(config_path, "w", encoding="utf-8") as f:
            json.dump(DEFAULT_CONFIG, f, indent=4)
        g_config_cache = DEFAULT_CONFIG
        g_config_mtime = config_path.stat().st_mtime

    if select:
        return g_config_cache.get(select)
    return g_config_cache

def set_config(select: str, value: Union[dict, str, bool, int, None]) -> tuple[bool, Optional[str]]:
    """
    设置配置信息 Set config info
    :param select: 配置项名称 Config item name
    :param value: 配置项值 Config item value
    :return: (设置是否成功, 消息) (Set success or not, message)
    """
    config_path = Path(DEFAULT_CONFIG_PATH)

    if not config_path.exists():
        with open(config_path, "w", encoding="utf-8") as f:
            json.dump(DEFAULT_CONFIG, f, indent=4)

    backup_success, backup_info = create_backup(config_path)
    if not backup_success:
        return False, f"备份失败: {backup_info}"

    with open(config_path, "r+", encoding="utf-8") as f:
        config = json.load(f)
        config[select] = value
        f.seek(0)
        json.dump(config, f, indent=4)
        f.truncate()

    with open(config_path, "r", encoding="utf-8") as f:
        verify_config = json.load(f)
        if verify_config.get(select) != value:
            shutil.move(backup_info, config_path)
            return False, f"配置项 '{select}' 设置失败，已恢复备份"

    global g_config_cache
    g_config_cache = None
    return True, f"配置项 '{select}' 设置成功"

@handle_config_exceptions
def reset_config() -> tuple[bool, Optional[str]]:
    """
    重置配置信息 Reset config info
    :return: (重置是否成功, 消息) (Reset success or not, message)
    """
    config_path = Path(DEFAULT_CONFIG_PATH)

    if config_path.exists():
        backup_success, backup_info = create_backup(config_path)
        if not backup_success:
            return False, f"备份失败: {backup_info}"

    with open(config_path, "w", encoding="utf-8") as f:
        json.dump(DEFAULT_CONFIG, f, indent=4)

    global g_config_cache
    g_config_cache = None
    return True, "配置已重置为默认值"



"""
py-logiliteal的格式化工具,用于格式化日志输出
py-logiliteal's formatter, used to format log output

"""
# encoding = utf-8
# python 3.13.5

from .configs import get_config
from typing import Any, Optional
from .time import get_asctime, get_time, get_weekday, get_date
from .styles import set_color, set_bg_color
import re

def fmt_level(level: str) -> int:
    """
    格式化日志级别
    Format log level
    :param level: 日志级别 Log level
    :return: 格式化后的日志级别 Formatted log level
    """
    level_map = {
        "DEBUG": 0,
        "INFO": 10,
        "WARN": 20,
        "ERRO": 30,
        "CRIT": 40,
        "UNKN": 50
    }
    return level_map.get(level.upper(), 50)

def fmt_level_number(level: int) -> str:
    """
    格式化日志级别数字
    Format log level number
    :param level: 日志级别数字 Log level number
    :return: 格式化后的日志级别 Formatted log level
    """
    if level < 10:
        return "DEBUG"
    elif level < 20:
        return "INFO"
    elif level < 30:
        return "WARN"
    elif level < 40:
        return "ERRO"
    elif level < 50:
        return "CRIT"
    else:
        return "UNKN"

def fmt_placeholder(message: Any, use_date_color: bool = True) -> str:
    """
    格式化占位符
    Format placeholder
    :param message: 消息内容 Message content
    :return: 格式化后的消息 Formatted message
    """
    class SafeDict(dict):
        def __missing__(self, key):
            return f'{{{key}}}'

    if not isinstance(message, str):
        message = str(message)
    if use_date_color:
        message = message.format_map(SafeDict(
            asctime = set_color(get_asctime(),"#28ffb6"),
            time = set_color(get_time(),"#28ffb6"),
            weekday = set_color(get_weekday(),"#28ffb6"),
            date = set_color(get_date(),"#28ffb6")
        ))
    else:
        message = message.format_map(SafeDict(
            asctime = get_asctime(),
            time = get_time(),
            weekday = get_weekday(),
            date = get_date(),
        ))
    return message

def fmt_message(message: Any, no_placeholder: bool = False, no_color: bool = False) -> str:
    """
    格式化消息内容
    Format message content
    :param message: 消息内容 Message content
    :return: 格式化后的消息 Formatted message
    """

    def process_color_tags(msg: str) -> str:
        from io import StringIO
        output = StringIO()
        stack = []
        last_end = 0
        pattern = re.compile(r'(<#([0-9a-fA-F]{6})>|</>)')
        current_color = None

        for match in pattern.finditer(msg):
            output.write(msg[last_end:match.start()])
            tag = match.group(1)
            last_end = match.end()

            if tag.startswith('<#'):
                stack.append(current_color)
                current_color = match.group(2)
            else:
                if stack:
                    current_color = stack.pop()
                else:
                    output.write(tag)

        output.write(msg[last_end:])
        result = output.getvalue()
        output.close()

        if current_color:
            result += ''.join(f'<#{color}>' for color in reversed(stack)) if stack else f'<#{current_color}>'
        return result
    if no_color:
        processed_message = str(message)
    else:
        processed_message = process_color_tags(str(message))
    if no_placeholder:
        return processed_message
    else:
        return process_color_tags(fmt_placeholder(processed_message)) if not no_color else fmt_placeholder(processed_message)

def fmt_level_name(level_name: str) -> str:
    if get_config("console_color") != True:
        return level_name
    level_name_nick_map = get_config("level_name")

    if level_name in level_name_nick_map:
        _lnn = level_name_nick_map[level_name]
        level_color_map = get_config("level_color")

        if level_name in level_color_map:
            if level_name == "DEBUG":
                return set_bg_color(set_color(_lnn, level_color_map[level_name]), "#34495e")
            return set_color(_lnn, level_color_map[level_name])
        return set_color(_lnn)
    return "UNKN"

def fmt_console(level: int, message: Any, prefix: str | None = None) -> Optional[str]:
    """
    格式化控制台输出
    Format console output
    :param level: 日志级别 Log level
    :param message: 消息内容 Message content
    :return: 格式化后的消息 Formatted message
    """
    console_level = get_config("console_level")
    if level != -1 and fmt_level(console_level) > level:
        return None
    fmt = get_config("console_format")
    prefix = prefix or ""
    return fmt_placeholder(fmt).format(
        levelname = fmt_level_name(fmt_level_number(level)),
        prefix = fmt_message(prefix, no_placeholder=True),
        message = fmt_message(message, no_placeholder=True)
    )

def fmt_file(level: int, message: Any, prefix: str | None = None) -> Optional[str]:
    """
    格式化文件输出
    Format file output
    :param level: 日志级别 Log level
    :param message: 消息内容 Message content
    :return: 格式化后的消息 Formatted message
    """
    fl = get_config("file_level")
    fmt = get_config("file_format")
    if fmt_level(fl) > level:
        return None
    if prefix is None:
        prefix = ""
    fmt = fmt_placeholder(fmt, use_date_color=False)
    return f"{fmt.format(
        levelname = fmt_level_number(level),
        prefix = fmt_message(prefix, no_placeholder=True, no_color=True),
        message = fmt_message(message, no_placeholder=True, no_color=True)
    )}\n"



"""
py-logiliteal - 简易,现代化具有色彩的日志记录器
py-logiliteal's config settings, used to set py-logiliteal's global config
"""
# encoding = utf-8
# python 3.13.5

from .utils import get_config, set_config, reset_config, create_backup
from .utils import get_asctime, get_date, get_time, get_weekday
from .utils import fmt_console, fmt_placeholder, fmt_message, fmt_level_name
from .utils import set_style, set_color, set_bg_color
from .levels import Logger

__all__ = [
    "get_config",
    "set_config",
    "reset_config",
    "get_asctime",
    "get_date",
    "get_time",
    "get_weekday",
    "fmt_console",
    "fmt_placeholder",
    "fmt_message",
    "fmt_level_name",
    "set_style",
    "set_color",
    "set_bg_color",
    "create_backup",
    "Logger" # 日志记录器非实例化
]


"""
py-logiliteal的样式工具,用于格式化日志输出
py-logiliteal's style tools, used to format log output

"""
# encoding = utf-8
# python 3.13.5

from typing import Union, Optional

def _get_hex_to_ansi(hex_color: str) -> Union[Optional[str], None]:
    """
    将16进制颜色值转换为ANSI转义序列
    Convert hex color value to ANSI escape sequence
    :param hex_color: 16进制颜色值 Hex color value
    :return: ANSI转义序列 ANSI escape sequence
    """
    if not hex_color.startswith("#"):
        return None
    r, g, b = int(hex_color[1:3], 16), int(hex_color[3:5], 16), int(hex_color[5:7], 16)
    return f"\033[38;2;{r};{g};{b}m"

def set_color(text: str, color: str) -> str:
    """
    设置文本颜色
    Set text color
    :param text: 文本内容 Text content
    :param color: 颜色值 Color value
    :return: 格式化后的文本 Formatted text
    """
    ansi = _get_hex_to_ansi(color)
    if not ansi:
        return text
    return f"{ansi}{text}\033[0m"

def set_bg_color(text: str, color: str) -> str:
    """
    设置文本背景颜色
    Set text background color
    :param text: 文本内容 Text content
    :param color: 颜色值 Color value
    :return: 格式化后的文本 Formatted text
    """
    ansi = _get_hex_to_ansi(color)
    if not ansi:
        return text
    # 将前景色ANSI代码转换为背景色代码 (38→48)
    ansi = ansi.replace("38;", "48;")
    return f"{ansi}{text}\033[0m"

def set_style(text: str, bold: bool = False, underline: bool = False, reverse: bool = False) -> str:
    """
    设置文本样式
    Set text style
    :param text: 文本内容 Text content
    :param bold: 是否加粗 Is bold
    :param underline: 是否下划线 Is underline
    :param reverse: 是否反相 Is reverse
    :return: 格式化后的文本 Formatted text
    """
    ansi = ""
    if bold:
        ansi += "\033[1m"
    if underline:
        ansi += "\033[4m"
    if reverse:
        ansi += "\033[7m"
    return f"{ansi}{text}\033[0m"



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="logiliteal",
    version="0.1.1",
    description="简洁,高扩展性,可自定义的日志库 / Simple, high extensibility, and customizable logging library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Nanaloveyuki/py-logiliteal",
    author="Nanaloveyuki",
    author_email="3541766758@qq.com",
    license="MIT",
    classifiers=[
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent",
    ],
    keywords="logging, color, format",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.12, <4",
    install_requires=[],
    project_urls={
        "Bug Reports": "https://github.com/Nanaloveyuki/py-logiliteal/issues",
        "Source": "https://github.com/Nanaloveyuki/py-logiliteal/",
    },
)




"""
工具函数
Utility functions
"""
# encoding = utf-8
# python 3.13.5

from .configs import get_config, set_config, reset_config, create_backup
from .time import get_asctime, get_date, get_time, get_weekday
from .fmt import fmt_console, fmt_placeholder, fmt_message, fmt_level_name
from .styles import set_color, set_bg_color, set_style

__all__ = [
    "get_config",
    "set_config",
    "reset_config",
    "create_backup",
    "get_asctime",
    "get_date",
    "get_time",
    "get_weekday",
    "fmt_console",
    "fmt_placeholder",
    "fmt_message",
    "fmt_level_name",
    "set_color",
    "set_bg_color",
    "set_style",
]
